
-- 1. Parse function

-- parse into datetime2
SELECT PARSE('Monday, 13 December 2010' AS datetime2 USING 'en-US') AS Result;

-- parse with currency symbol
SELECT PARSE('345,98' AS money USING 'de-DE') AS Result;

-- The English language is mapped to en-US specific culture
SET LANGUAGE 'English'
GO
SELECT PARSE('12/16/2010' AS datetime2) AS Result

-- 2. TRY_PARSE function

-- Returns NULL if it doesn't work
SELECT TRY_PARSE('Jabberwokkie' AS datetime2 USING 'en-US') AS Result
-- vs.
SELECT PARSE('Jabberwokkie' AS datetime2 USING 'en-US') AS Result

-- Detect NULLs with TRY_PARSE
SELECT
    CASE WHEN TRY_PARSE('Aragorn' AS decimal USING 'sr-Latn-CS') IS NULL
        THEN 'True'
        ELSE 'False'
END
AS Result

-- Using IIF with TRY_PARSE and implicit culture setting
SET LANGUAGE English
SELECT IIF(TRY_PARSE('01/01/2011' AS datetime2) IS NULL, 'True', 'False') AS Result

-- 3. TRY_CONVERT function

-- returns NULL if convert fails
SELECT 
    CASE WHEN TRY_CONVERT(float,'test') IS NULL 
    THEN 'Cast failed'
    ELSE 'Cast succeeded'
END AS Result
GO

-- Explicit conversion from data type int to xml is not allowed
CREATE FUNCTION dbo.TestFunction()
RETURNS XML
AS
BEGIN
    RETURN TRY_CONVERT(xml, 4) 
END
GO

-- Try_convert succeeds
SELECT TRY_CONVERT(datetime2, '12/31/2010') AS Result

-- How about TRY_CAST?
SELECT TRY_CAST('12/31/2010' as datetime2) as Result

-- 4. CHOOSE function
SELECT CHOOSE ( 3, 'Manager', 'Director', 'Developer', 'Tester' ) AS Result;


-- 5. IIF Function
DECLARE @a int = 45;
DECLARE @b int = 40;
SELECT IIF ( @a > @b, 'TRUE', 'FALSE' ) AS Result;

-- IIF with NULLs, statement fails
SELECT IIF ( 45 > 30, NULL, NULL ) AS Result;

-- IIF with NULL parameters
DECLARE @P INT = NULL;
DECLARE @S INT = NULL;
SELECT IIF ( 45 > 30, @p, @s ) AS Result;

-- 6. CONCAT function

-- Simple CONCAT
SELECT CONCAT ( 'Happy', 'Christmas', 12, '/', '25' ) AS Result;
GO

-- CONCAT with NULL values
CREATE TABLE #temp (
    emp_name nvarchar(200) NOT NULL,
    emp_middlename nvarchar(200) NULL,
    emp_lastname nvarchar(200) NOT NULL
);
INSERT INTO #temp VALUES( 'Name', NULL, 'Lastname' );
GO
SELECT CONCAT( emp_name, emp_middlename, emp_lastname ) AS Result
FROM #temp;

-- 7. FORMAT function

-- Simple FORMAT
DECLARE @d DATETIME = '01/01/2011';
SELECT FORMAT ( @d, 'd', 'en-US' ) AS Result;
GO

-- Custom format strings
DECLARE @d DATETIME = GETDATE();
SELECT FORMAT( @d, 'dd/MM/yyyy', 'en-US' ) AS Result
GO

DECLARE @d DATETIME = GETDATE();
SELECT FORMAT( @d, 'dd MM yyyy', 'en-US' ) AS Result
GO



