
use tempdb;
go

--1. Using THROW to raise an exception
--The following example shows how use the THROW statement to raise an exception.
	 
THROW 51000, 'The row does not exist.', 1

--2. Using THROW to reraise a system error
--The following example shows how use the THROW statement to raise the last thrown exception again.
	 
USE tempdb;
GO
CREATE TABLE dbo.TestRethrow(
    ID INT PRIMARY KEY
);
BEGIN TRY
    INSERT dbo.TestRethrow(ID) VALUES(1);
--  Force error 2627, Violation of PRIMARY KEY constraint to be raised.
    INSERT dbo.TestRethrow(ID) VALUES(1);
END TRY
BEGIN CATCH
    PRINT 'In Catch Block.';
    THROW;
END CATCH;
PRINT 'Entered the catch block.'; 
GO

DROP TABLE dbo.TestRethrow

--Msg 2627, Level 14, State 1, Line 1 
--Violation of PRIMARY KEY constraint 'PK__TestReth__3214EC272E3BD7D3'. Cannot insert duplicate key in object 'dbo.TestRethrow'. 
--The statement has been terminated. 

--3. Using FORMATMESSAGE with THROW
--The following example shows how to use the FORMATMESSAGE statement with THROW to throw a message string.
	 
EXEC sp_addmessage 
    @msgnum = 60000, 
    @severity = 16,
    @msgtext = N'This is a test message with one string parameter (%s)',
    @lang = 'us_english';

-- Statement before THROW MUST end with a semicolon 
DECLARE @Message NVARCHAR(2048);
SELECT @Message = FORMATMESSAGE(60000, 'My String'); -- semicolon required here
THROW 60000, @Message, 1  
GO

EXEC sp_dropmessage 60000
go
