
--Using EXECUTE to redefine a single result set

USE AdventureWorks2008R2;
GO

-- Uses ToString() to format hierarchyid
EXEC sp_helptext 'uspGetEmployeeManagers';

-- 1. Use with resultsets when executing sproc to cause implicit data type conversion
EXEC uspGetEmployeeManagers 16;
GO

EXEC uspGetEmployeeManagers 16
WITH RESULT SETS
( 
   ([Reporting Level] int NOT NULL,
    [ID of Employee] int NOT NULL,
    [Employee First Name] nvarchar(50) NOT NULL,
    [Employee Last Name] nvarchar(50) NOT NULL,
	-- hierarchyid as string, implicit conversion
	-- can also do collation conversion
    [Employee ID of Manager] nvarchar(50)  NOT NULL, 
    [Manager First Name] nvarchar(50) NOT NULL,
    [Manager Last Name] nvarchar(50) NOT NULL )
);

EXEC uspGetEmployeeManagers 16
WITH RESULT SETS
( 
   ([Reporting Level] int NOT NULL,
    [ID of Employee] int NOT NULL,
    [Employee First Name] nvarchar(50) NOT NULL,
    [Employee Last Name] nvarchar(50) NOT NULL,
	-- no conversion
    [Employee ID of Manager] hierarchyid  NOT NULL,
    [Manager First Name] nvarchar(50) NOT NULL,
    [Manager Last Name] nvarchar(50) NOT NULL )
);

-- 2. Using EXECUTE to redefine a two result sets
USE AdventureWorks2008R2;
GO

--Create the procedure
CREATE PROC Production.ProductList @ProdName nvarchar(50)
AS
-- First result set
SELECT ProductID, Name, ListPrice
    FROM Production.Product
    WHERE Name LIKE @ProdName;
-- Second result set 
SELECT Name, COUNT(S.ProductID) AS NumberOfOrders
    FROM Production.Product AS P
    JOIN Sales.SalesOrderDetail AS S
        ON P.ProductID  = S.ProductID 
    WHERE Name LIKE @ProdName
    GROUP BY Name;
GO

-- Execute the procedure 
EXEC Production.ProductList '%Frame%'
WITH RESULT SETS 
(
    (ProductID int,   -- first result set definition starts here
    Name Name,        -- Name is a UDDT
    ListPrice money)
    ,                 -- comma separates result set definitions
    (Name Name,       -- second result set definition starts here
    NumberOfOrders int)
);

/* No
CREATE TYPE Test AS TABLE
   ([Reporting Level] int NOT NULL,
    [ID of Employee] int NOT NULL,
    [Employee First Name] nvarchar(50) NOT NULL,
    [Employee Last Name] nvarchar(50) NOT NULL,
	-- no conversion
    [Employee ID of Manager] hierarchyid  NOT NULL,
    [Manager First Name] nvarchar(50) NOT NULL,
    [Manager Last Name] nvarchar(50) NOT NULL )

EXEC uspGetEmployeeManagers 16
WITH RESULT SETS
( T dbo.Test)
*/